@@ -29,7 +29,7 @@
 #include "mainwin.h"
 
 
-#define NOPTS 6
+#define NOPTS 7
 #define CP (char *)
 
 
@@ -40,7 +40,8 @@ XrmOptionDescRec options [NOPTS] =
     {CP"-p", CP".presets",  XrmoptionSepArg,  CP"."    },
     {CP"-V", CP".volume",   XrmoptionSepArg,  CP"-60"  },
     {CP"-g", CP".geometry", XrmoptionSepArg,  0        },
-    {CP"-s", CP".server",   XrmoptionSepArg,  0        }
+    {CP"-s", CP".server",   XrmoptionSepArg,  0        },
+    {CP"-o", CP".outputs",  XrmoptionSepArg,  0        }
 };
 
 
