@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2006-2011 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2006-2017 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -24,6 +24,7 @@
 #include "decoder.h"
 
 
+
 const float Decoder::_g_norm [16] = { 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
 
 
@@ -106,35 +107,39 @@ void Decoder::init (int frate)
 void Decoder::reset (void)
 {
     int      i;
-    Output  *X;
+    Output  *Z;
 
     // Reset everything.
     memset (_sig0, 0, FRAG_SIZE * sizeof (float));
     memset (_sig1, 0, FRAG_SIZE * sizeof (float));
     memset (_sig2, 0, FRAG_SIZE * sizeof (float));
     memset (_sig3, 0, FRAG_SIZE * sizeof (float));
-    for (i = 0, X = _output; i < MAXOP; i++, X++)
+    memset (_sig4, 0, FRAG_SIZE * sizeof (float));
+    for (i = 0, Z = _output; i < MAXOP; i++, Z++)
     {
-	memset (X->_dline, 0, _dsize * sizeof (float)); 
-	X->_delay = 0;
-	X->_gcorr = 1.0f;
-	X->_gain0 = 0.0f;
-	X->_nffilt1.reset ();
-	X->_nffilt2.reset ();
-	X->_nffilt3.reset ();
+	memset (Z->_dline, 0, _dsize * sizeof (float)); 
+	Z->_delay = 0;
+	Z->_gcorr = 1.0f;
+	Z->_gain0 = 0.0f;
+	Z->_nffilt1.reset ();
+	Z->_nffilt2.reset ();
+	Z->_nffilt3.reset ();
+	Z->_nffilt4.reset ();
     }
     for (i = 0; i < 3; i++) _nffilt1 [i].reset ();
     for (i = 0; i < 5; i++) _nffilt2 [i].reset ();
     for (i = 0; i < 7; i++) _nffilt3 [i].reset ();
-    for (i = 0; i < 16; i++)
+    for (i = 0; i < 9; i++) _nffilt4 [i].reset ();
+    for (i = 0; i < MAXIP; i++)
     {
-        _xover [i].reset ();
+        _lr2filt [i].reset ();
 	_scale [i] = 0;
     }
     _nsig0 = 0;
     _nsig1 = 0;
     _nsig2 = 0;
     _nsig3 = 0;
+    _nsig4 = 0;
     _noutp = 0;
     _nband = 1;
     _comp  = 0;
@@ -172,20 +177,18 @@ void Decoder::set_config (AD_conf *C)
         if (m & 1)
 	{
             _scale [j++] = B [i];
-	    if      (i < 1) _nsig0++;
-	    else if (i < 4) _nsig1++;
-	    else if (i < 9) _nsig2++;
-	    else            _nsig3++;
+	    if      (i < 1)  _nsig0++;
+	    else if (i < 4)  _nsig1++;
+	    else if (i < 9)  _nsig2++;
+	    else if (i < 16) _nsig3++;
+	    else             _nsig4++;
 	}    
     }
 
     // Set crossover filter frequency.
     if (_nband > 1)
     {
-	for (i = 0; i < _nsig0 + _nsig1 + _nsig2 + _nsig3; i++)
-	{
-            _xover [i].init (C->_opt.xfreq / _frate);
-	}
+        _lr2conf.init (C->_opt.xfreq / _frate);
     }
 
     // Find maximum and average speaker distance.
@@ -299,7 +302,7 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
             p2 = _ip_lf [k];
 	    p3 = _ip_hf [k];
 	    for (i = 0; i < n; i++) p2 [i] = _scale [k] * p1 [i];
-	    if (_nband > 1) _xover [k].process (n, p2, p2, p3);
+	    if (_nband > 1) _lr2filt [k].process (&_lr2conf, n, p2, p2, p3);
 	    ip [k++] += n;
 	}
 	// Scale, filter and bandsplit first degree.
@@ -310,7 +313,7 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
             p3 = &(_ip_hf [k][0]);
 	    if (_comp & COMP_NFE_IP) _nffilt1 [j].process (n, p1, p2, _scale [k]);
             else for (i = 0; i < n; i++) p2 [i] = _scale [k] * p1 [i];
-  	    if (_nband > 1) _xover [k].process (n, p2, p2, p3);
+	    if (_nband > 1) _lr2filt [k].process (&_lr2conf, n, p2, p2, p3);
  	    ip [k++] += n;
 	}	    
 	// Scale, filter and bandsplit second degree.
@@ -321,7 +324,7 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
             p3 = &(_ip_hf [k][0]);
 	    if (_comp & COMP_NFE_IP) _nffilt2 [j].process (n, p1, p2, _scale [k]);
             else for (i = 0; i < n; i++) p2 [i] = _scale [k] * p1 [i];
-  	    if (_nband > 1) _xover [k].process (n, p2, p2, p3);
+	    if (_nband > 1) _lr2filt [k].process (&_lr2conf, n, p2, p2, p3);
  	    ip [k++] += n;
 	}	    
 	// Scale, filter and bandsplit third degree.
@@ -332,7 +335,18 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
             p3 = &(_ip_hf [k][0]);
 	    if (_comp & COMP_NFE_IP) _nffilt3 [j].process (n, p1, p2, _scale [k]);
             else for (i = 0; i < n; i++) p2 [i] = _scale [k] * p1 [i];
-  	    if (_nband > 1) _xover [k].process (n, p2, p2, p3);
+	    if (_nband > 1) _lr2filt [k].process (&_lr2conf, n, p2, p2, p3);
+ 	    ip [k++] += n;
+	}	    
+	// Scale, filter and bandsplit degree 4 inputs.
+	for (j = 0; j < _nsig4; j++)
+	{
+	    p1 = ip [k];
+            p2 = &(_ip_lf [k][0]);
+            p3 = &(_ip_hf [k][0]);
+	    if (_comp & COMP_NFE_IP) _nffilt4 [j].process (n, p1, p2, _scale [k]);
+            else for (i = 0; i < n; i++) p2 [i] = _scale [k] * p1 [i];
+	    if (_nband > 1) _lr2filt [k].process (&_lr2conf, n, p2, p2, p3);
  	    ip [k++] += n;
 	}	    
 
@@ -352,6 +366,7 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
 	// Loop over all outputs.
 	for (j = 0, Z = _output; j < _noutp; j++, Z++)
 	{
+	    if (op [j] == 0) continue;
 	    k = 0;
 	    if (_nsig0)
 	    { 
@@ -384,6 +399,14 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
 		else for (i = 0; i < n; i++) _sig0 [i] += _sig3 [i];
 	        k += _nsig3;
 	    }
+	    if (_nsig4)
+	    { 
+  	        if (_nband > 1) matr2band (n, k, k + _nsig4, _sig4, Z);
+	        else            matr1band (n, k, k + _nsig4, _sig4, Z);
+	        if (_comp & COMP_NFE_OP) Z->_nffilt4.process_add (n, _sig4, _sig0, 1);
+		else for (i = 0; i < n; i++) _sig0 [i] += _sig4 [i];
+	        k += _nsig4;
+	    }
 
 	    // Mute, solo and volume logic.
 	    g0 = Z->_gain0;
@@ -405,9 +428,9 @@ void Decoder::process (int nf, float *ip[], float *op[], float *ts, float *lm)
 	    {
 		g0 += d;
   	        p2 [i] = t = g0 * p1 [i];
-		s += t * t;
+		t = fabsf (t);
+		if (t > s) s = t;
 	    }
-	    s = sqrtf (2 * s / n);
 
 	    // Update level measurement.
 	    lm [j] = 0.98f * lm [j] + 1e-10f;
@@ -459,13 +482,11 @@ void Decoder::matr1band (int nf, int ind0, int ind1, float *out, Output *Z)
     a = Z->_matlf [ind0];
     p = &_ip_lf [ind0][0];
     for (i = 0; i < nf; i++) out [i] = a * p [i];
-    ind0++;
-    while (ind0 < ind1)
+    while (++ind0 < ind1)
     {
         a = Z->_matlf [ind0];
         p = &_ip_lf [ind0][0];
         for (i = 0; i < nf; i++) out [i] += a * p [i];
-	ind0++;
     }
 }
 
@@ -480,14 +501,14 @@ void Decoder::matr2band (int nf, int ind0, int ind1, float *out, Output *Z)
     p = &_ip_lf [ind0][0];
     q = &_ip_hf [ind0][0];
     for (i = 0; i < nf; i++) out [i] = a * p [i] + b * q [i];
-    ind0++;
-    while (ind0 < ind1)
+    while (++ind0 < ind1)
     {
         a = Z->_matlf [ind0];
         b = Z->_mathf [ind0];
         p = &_ip_lf [ind0][0];
         q = &_ip_hf [ind0][0];
         for (i = 0; i < nf; i++) out [i] += a * p [i] + b * q [i];
-	ind0++;
     }
 }
+
+
