@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2006-2011 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2006-2017 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -37,13 +37,14 @@ public:
     Output (void) : _dline (0) {}
     ~Output (void) { delete[] _dline; }
 
-    float        _matlf [16];
-    float        _mathf [16];
-    NF_filt1     _nffilt1;
+    float        _matlf [MAXIP]; // matrix row
+    float        _mathf [MAXIP]; // matrix row
+    NF_filt1     _nffilt1;       // near field filters
     NF_filt2     _nffilt2;
     NF_filt3     _nffilt3;
-    int          _delay;
-    float       *_dline;
+    NF_filt4     _nffilt4;
+    int          _delay;         // delay in samples  
+    float       *_dline;         // delay buffer
     float        _gcorr;
     float        _gain0;
 };
@@ -83,6 +84,7 @@ private:
     int         _nsig1;                      // number of inputs of degree 1
     int         _nsig2;                      // number of inputs of degree 2
     int         _nsig3;                      // number of inputs of degree 3
+    int         _nsig4;                      // number of inputs of degree 4
     int         _noutp;                      // number of outputs
     int         _nband;                      // number of frequency bands, 1 or 2
     int         _dsize;                      // size of delay lines in samples
@@ -95,22 +97,25 @@ private:
     float       _gain;                       // volume
     int         _idel;                       // delay write index 
     int         _itst;                       // test read index
-    NF_filt1    _nffilt1 [3];
+    NF_filt1    _nffilt1 [3];                // near_field filters
     NF_filt2    _nffilt2 [5];
     NF_filt3    _nffilt3 [7];
-    Xover2      _xover [16];
-    float       _scale [16]; 
-    float       _ip_lf [16][FRAG_SIZE];
-    float       _ip_hf [16][FRAG_SIZE];
+    NF_filt4    _nffilt4 [9];
+    LR2conf     _lr2conf;
+    LR2filt     _lr2filt [MAXIP];
+    float       _scale [MAXIP]; 
+    float       _ip_lf [MAXIP][FRAG_SIZE];
+    float       _ip_hf [MAXIP][FRAG_SIZE];
     Output      _output [MAXOP];
     float       _sig0 [FRAG_SIZE];
     float       _sig1 [FRAG_SIZE];
     float       _sig2 [FRAG_SIZE];
     float       _sig3 [FRAG_SIZE];
+    float       _sig4 [FRAG_SIZE];
     float      *_test;
 
-    static const float _g_norm [16];
-    static const float _g_sn3d [16];
+    static const float _g_norm [MAXIP];
+    static const float _g_sn3d [MAXIP];
     static const float _g_fuma [16];
 };
 
