@@ -23,15 +23,6 @@
 #include "jclient.h"
 
 
-static const char *ipnames [MAXIP] =
-{
-    "0w",
-    "1y", "1z", "1x",
-    "2v", "2t", "2r", "2s", "2u",
-    "3q", "3o", "3m", "3k", "3l", "3n", "3p"
-};
-
-
 Jclient::Jclient (const char *jname, const char *jserv) :
     A_thread ("Jclient"),
     _jack_client (0),
@@ -40,6 +31,7 @@ Jclient::Jclient (const char *jname, const char *jserv) :
     _state (ST_BYPASS),
     _n_inp (0),
     _n_out (0),
+    _n_act (0),
     _gain (0),
     _mute (0),
     _solo (0),
@@ -86,7 +78,7 @@ void Jclient::init_jack (const char *jname, const char *jserv)
     _fsize = jack_get_buffer_size (_jack_client);
     for (i = 0; i < MAXIP; i++)
     {
-	sprintf (s, "in.%s", ipnames [i]);
+	snprintf (s, 16, "in.%d", i);
         _jack_ipports [i] = jack_port_register (_jack_client, s, JACK_DEFAULT_AUDIO_TYPE, JackPortIsInput, 0);
 	_ipmap [i] = 0;
     }
@@ -122,11 +114,15 @@ int Jclient::jack_static_process (jack_nframes_t nframes, void *arg)
 
 int Jclient::jack_process (jack_nframes_t nframes)
 {
-    unsigned int   i, s;
+    unsigned int   i, n, s;
     int            e;
     float          *tip;
     float          *ipp [MAXIP];
     float          *opp [MAXOP];
+#ifdef ENABLE_UHJ_INPUT
+    float          *uhj_L, *uhj_R;
+    float          sr, si, dr, di;
+#endif
 
     s = _state;
     e = get_event_nowait ();
@@ -139,23 +135,29 @@ int Jclient::jack_process (jack_nframes_t nframes)
 
     if (_state != ST_BYPASS)
     {
-        for (i = 0; i < _n_out; i++) opp [i] = (float *) jack_port_get_buffer (_jack_opports [i], nframes);
+	n = 0;
+	memset (opp, 0, MAXOP * sizeof (float *));
+        for (i = 0; i < _n_out; i++)
+	{
+	    opp [i] = (float *) jack_port_get_buffer (_jack_opports [i], nframes);
+	}
 	if (_state == ST_PROCESS)
 	{
- 	    tip = (float *) jack_port_get_buffer (_jack_testip, nframes);
   	    for (i = 0; i < _n_inp; i++)
 	    {
-                ipp [i] = (float *) jack_port_get_buffer (_ipmap [i], nframes);
-	    }
+                ipp [i] = (float *) jack_port_get_buffer (_ipmap [i], nframes);	 
+            }
+ 	    tip = (float *) jack_port_get_buffer (_jack_testip, nframes);
 	    _decoder.set_gain (s == ST_PROCESS ? _gain : 0);
 	    _decoder.set_mute (_mute);
 	    _decoder.set_solo (_solo);
 	    _decoder.set_test (_test);
-	    _decoder.process (nframes, ipp, opp, tip, _level);
+            _decoder.process (nframes, ipp, opp, tip, _level);
+	    n = _n_act;
 	}
-	else
+	for (i = n; i < _n_out; i++)
 	{
-	    for (i = 0; i < _n_out; i++) memset (opp [i], 0, nframes * sizeof (float));
+	    memset (opp [i], 0, nframes * sizeof (float));
 	}
     }
 
@@ -171,6 +173,7 @@ int Jclient::jack_process (jack_nframes_t nframes)
 void Jclient::set_config (AD_conf *C)
 {
     _decoder.set_config (C);
+    _n_act = C->_dec.nspkr;
 }
 
 
@@ -202,7 +205,22 @@ void Jclient::create_opports (AD_conf *C)
     _n_out = C->_dec.nspkr;
     for (i = 0; i < _n_out; i++)
     {
-	sprintf (s, "out_%s", C->_speakers [i]._label);
+	snprintf (s, 64, "out_%s", C->_speakers [i]._label);
+        _jack_opports [i] = jack_port_register (_jack_client, s, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+    }
+}
+
+
+void Jclient::create_opports (int n)
+{
+    unsigned int i;
+    char s [64];
+
+    delete_opports ();
+    _n_out = n;
+    for (i = 0; i < _n_out; i++)
+    {
+	snprintf (s, 64, "out_%d", i + 1);
         _jack_opports [i] = jack_port_register (_jack_client, s, JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
     }
 }
@@ -223,12 +241,12 @@ void Jclient::delete_opports (void)
 void Jclient::connect_opports (AD_conf *C)
 {
     unsigned int i;
-    char s [64];
+    char s [256];
 
     disconn_opports ();
     for (i = 0; i < _n_out; i++)
     {
-	sprintf (s, "%s:out_%s", _jname, C->_speakers [i]._label);
+	snprintf (s, 256, "%s:out_%s", _jname, C->_speakers [i]._label);
 	jack_connect (_jack_client, s, C->_speakers [i]._jackp);
     }
 }
