@@ -59,6 +59,25 @@ Mainwin::Mainwin (X_rootwin *parent, X_resman *xres, int xp, int yp, Jclient *jc
 
     makedisp ();
 
+    p = xres->get (".outputs", "0");
+    if (p || *p)
+    {
+	_nout = atoi (p);
+	if (_nout < 0) _nout = 0;
+	if (_nout > MAXOP) _nout = MAXOP;
+    }
+    if (_nout)
+    {
+	_fixop = true;
+	_nact = _nout;
+	jclient->create_opports (_nout);
+    }    
+    else
+    {
+	_fixop = false;
+	_nact = _nout = _conf._dec.nspkr;
+    }
+    
     p = xres->get (".volume", "-80");
     if (p || *p)
     {
@@ -130,7 +149,7 @@ void Mainwin::handle_time (void)
     const float  *level;
 
     level = _jclient->get_level ();
-    for (i = 0; i < _conf._dec.nspkr; i++) _dlev [i]->update (level [i]);
+    for (i = 0; i < _nact; i++) _dlev [i]->update (level [i]);
     inc_time (50000);
     XFlush (dpy ());
 }
@@ -227,12 +246,24 @@ void Mainwin::applconf (void)
     syncdec (EV_GO_SILENCE);
     if (_conf._state & AD_conf::ST_OUTS)
     {
+	_nact = _conf._dec.nspkr;
+	if (_fixop)
+	{
+	    if (_nact > _nout) _nact = _nout;
+	}
+	else
+	{
+	    _nout = _nact;
+	}
         confdisp ();
-        _jclient->disconn_opports ();
-        syncdec (EV_GO_BYPASS);
-        _jclient->create_opports (&_conf);
-        syncdec (EV_GO_SILENCE);
-        _jclient->connect_opports (&_conf);
+	if (!_fixop)
+	{
+            _jclient->disconn_opports ();
+            syncdec (EV_GO_BYPASS);
+            _jclient->create_opports (&_conf);
+            syncdec (EV_GO_SILENCE);
+            _jclient->connect_opports (&_conf);
+	}
     }
     if (_conf._state & AD_conf::ST_INPS) _jclient->set_ipports (&_conf);
     if (_conf._state & AD_conf::ST_OPTS) _jclient->set_config (&_conf);
@@ -251,6 +282,7 @@ void Mainwin::syncdec (int e)
 
 void Mainwin::makedisp (void)
 {
+    char s [64];
     int  i, x, x0, y0;
 
     x0 = 120;
@@ -271,6 +303,8 @@ void Mainwin::makedisp (void)
     {
         _dlev [i] = new Meter (_disp, x, 10);
         _btst [i] = new X_tbutton (this, this, &Bst0, x0 + x - 14, y0 + ((i & 1) ? 208 : 189), 0, 0, i);
+	sprintf (s, "%d", i + 1);
+	_btst [i]->set_text (s, 0);
         x += 20;
     }
     _scrt = new X_vscale (_disp, &Sdb1, x, 8, 15, 0);
@@ -300,11 +334,10 @@ void Mainwin::makedisp (void)
 
 void Mainwin::confdisp (void)
 {
-    int      i, n;
+    int      i;
     X_hints  H;
 
-    n = _conf._dec.nspkr;
-    _xd = 60 + n * 20;
+    _xd = 60 + _nout * 20;
     _scrt->x_move (_xd - 27, 8);
     _disp->x_resize (_xd, _yd);
 
@@ -316,25 +349,30 @@ void Mainwin::confdisp (void)
     H.maxsize (_xm, _ym);
     x_apply (&H);
 
-    for (i = 0; i < n; i++)
+    for (i = 0; i < _nout; i++)
     {
-        _btst [i]->set_text (_conf._speakers [i]._label, 0);
-        _btst [i]->redraw ();
+	if (!_fixop)
+	{
+            _btst [i]->set_text (_conf._speakers [i]._label, 0);
+            _btst [i]->redraw ();
+	}
         _btst [i]->set_stat (0);
+        _dlev [i]->update (0.0f);
         _btst [i]->x_map ();
         _dlev [i]->x_map ();
     }
-    for (i = n; i < MAXOP; i++)
+    for (i = _nout; i < MAXOP; i++)
     {
 	_btst [i]->x_unmap ();
 	_dlev [i]->x_unmap ();
     } 
-    _jclient->clr_mute (~0LL);
-    _jclient->clr_solo (~0LL);
-    _jclient->set_test (0);
     _bintt->set_stat (0);
     _bextt->set_stat (0);
     _bup->x_map ();
+
+    _jclient->clr_mute (~0LL);
+    _jclient->clr_solo (~0LL);
+    _jclient->set_test (0);
 }
 
 
