@@ -60,6 +60,7 @@ private:
     void applconf (void);
     void editconf (void);
     void makedisp (void);
+    void reset (void);
     void confdisp (void);
     void drawdisp (void);
     void testbutt (int b);
@@ -81,6 +82,9 @@ private:
     X_tbutton      *_bextt;
     X_ibutton      *_bup;
     X_ibutton      *_bdn;
+    bool            _fixop;
+    int             _nout;
+    int             _nact;
     int             _xm, _ym;
     int             _xd, _yd;
     AD_conf         _conf;
