@@ -1,6 +1,6 @@
 // ----------------------------------------------------------------------------
 //
-//  Copyright (C) 2006-2011 Fons Adriaensen <fons@linuxaudio.org>
+//  Copyright (C) 2006-2017 Fons Adriaensen <fons@linuxaudio.org>
 //    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
@@ -86,4 +86,25 @@ private:
 };
 
 
+class NF_filt4
+{
+public:
+
+    NF_filt4 (void) {}
+    ~NF_filt4 (void) {}
+
+    void init (float w);
+    void init (NF_filt4& F);
+    void reset (void) { _z1 = _z2 = _z3 = _z4 = 0; }
+    void process (int n, float *ip, float *op, float gain);
+    void process_add (int n, float *ip, float *op, float gain);
+
+private:
+
+    float _g;
+    float _d1, _d2, _d3, _d4;
+    float _z1, _z2, _z3, _z4;
+};
+
+
 #endif
