@@ -1,10 +1,10 @@
-// ----------------------------------------------------------------------------
+//  ------------------------------------------------------------------------
+//
+//  Copyright (C) 2009-2017 Fons Adriaensen <fons@linuxaudio.org>
 //
-//  Copyright (C) 2006-2011 Fons Adriaensen <fons@linuxaudio.org>
-//    
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License as published by
-//  the Free Software Foundation; either version 2 of the License, or
+//  the Free Software Foundation; either version 3 of the License, or
 //  (at your option) any later version.
 //
 //  This program is distributed in the hope that it will be useful,
@@ -13,10 +13,9 @@
 //  GNU General Public License for more details.
 //
 //  You should have received a copy of the GNU General Public License
-//  along with this program; if not, write to the Free Software
-//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 //
-// ----------------------------------------------------------------------------
+//  ------------------------------------------------------------------------
 
 
 #include <math.h>
@@ -24,14 +23,14 @@
 
 
 #define P2F 6.283185f
+#define EPS 1e-20f
 
 
-
-void Xover2::init (float fmid)
+void LR2conf::init (float f)
 {
     float w, c, s;
 
-    w = P2F * fmid;
+    w = P2F * f;
     c = cosf (w);
     s = sinf (w);
     _c0 = (c < 1e-3f) ? (-0.5f * c) : ((s - 1) / c);
@@ -39,32 +38,60 @@ void Xover2::init (float fmid)
 }
 
 
-void Xover2::process (int k, float *in, float *lp, float *hp)
+void LR2filt::process (const LR2conf *C, int k,
+		       const float *in, float *lp, float *hp)
 {
     float d, t, x0, x1, z0, z1, z2;
 
+    // Get filter state.
     z0 = _z0;
     z1 = _z1;
     z2 = _z2;
-   
+    // Run filter for n samples.
     while (k--)
     {
-	x1 = x0 = *in++;
-        d = _c1 * (x1 - z1) + 1e-20f;
+	x1 = x0 = *in++ + EPS;
+        d = C->_c1 * (x1 - z1);
         x1 = z1 + d;
         z1 = x1 + d;      
-        d = _c1 * (x1 - z2) + 1e-20f;
+        d = C->_c1 * (x1 - z2);
         x1 = z2 + d;
         z2 = x1 + d;      
         *lp++ = x1;
-        t = x0 - _c0 * z0;
-        x0 = z0 + _c0 * t;        
-        z0 = t + 1e-20f;
+        t = x0 - C->_c0 * z0;
+        x0 = z0 + C->_c0 * t;        
+        z0 = t;
 	*hp++ = x0 - x1;
     }
-
+    // Save filter state.
     _z0 = z0;
     _z1 = z1;
     _z2 = z2;
 }
 
+
+void AP1conf::init (float f)
+{
+    float w, c, s;
+
+    w = P2F * f;
+    c = cosf (w);
+    s = sinf (w);
+    _c0 = (c < 1e-3f) ? (-0.5f * c) : ((s - 1) / c);
+}
+
+
+void AP1filt::process (const AP1conf *C, int k, float *sig)
+{
+    float t, z0;
+
+    z0 = _z0;
+    while (k--)
+    {
+        t = *sig - C->_c0 * z0 + EPS;
+        *sig++ = z0 + C->_c0 * t;        
+        z0 = t;
+    }
+    _z0 = z0;
+}
+
